library(dplyr)  
library(readxl)
library(panelr)
library(plm)
library(DescTools)
library(pdynmc)
library(sandwich)
library(lmtest)
library(car)

FY2013 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2013.xlsx')
FY2014 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2014.xlsx')
FY2015 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2015.xlsx')
FY2016 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2016.xlsx')
FY2017 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2017.xlsx')
FY2018 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2018.xlsx')
FY2019 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2019.xlsx')
FY2020 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2020.xlsx')
FY2021 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2021.xlsx')
FY2022 <- read_excel('C:/Users/Voicu Dragomir/OneDrive/Research/ARTICLE. Knowledge capital/scriptdb/FY2022.xlsx')

FYALL <- left_join(FY2022, FY2021, by = "RIC") 
FYALL <- left_join(FYALL, FY2020, by = "RIC") 
FYALL <- left_join(FYALL, FY2019, by = "RIC") 
FYALL <- left_join(FYALL, FY2018, by = "RIC") 
FYALL <- left_join(FYALL, FY2017, by = "RIC") 
FYALL <- left_join(FYALL, FY2016, by = "RIC") 
FYALL <- left_join(FYALL, FY2015, by = "RIC") 
FYALL <- left_join(FYALL, FY2014, by = "RIC") 
FYALL <- left_join(FYALL, FY2013, by = "RIC") # 625 companies, 6250 obs. balanced / 19,780 obs. total

LONG22 <- long_panel(FYALL, prefix = "_FY", begin = 2013, end = 2022, label_location = "end")

# !! Check for negative values as errors and replace with NAs __________________

# BEP = EBIT / TA
Desc(LONG22$TA) # correct, no negative values
LONG22 <- LONG22 %>%
  mutate(BEP = EBIT / TA)

# ROA = NIBEI / TA
LONG22$ROA <- LONG22$NIBEI / LONG22$TA

# SCR = INT / TFA
Desc(LONG22$INT) 
LONG22$INT <- if_else(LONG22$INT < 0, NA, LONG22$INT) # Replace negative values with NA

Desc(LONG22$TFA)
LONG22$TFA <- if_else(LONG22$TFA <= 0, NA, LONG22$TFA) # Replace negative values with NA

LONG22 <- LONG22 %>%
  mutate(SCR = INT / TFA)

# LTA = log(TA)
LONG22$LTA <- log(LONG22$TA)

# LFT = log(FTO)
Desc(LONG22$FTO) # check negatives
LONG22$FTO <- if_else(LONG22$FTO <= 0, NA, LONG22$FTO)
LONG22$LFT <- log(LONG22$FTO)

# WCR = WCA / TA
LONG22$WCR <- LONG22$WCA / LONG22$TA

# LEV = TD / TE
Desc(LONG22$TD) # check negatives / no negatives, correct
Desc(LONG22$TE) # check negatives

LONG22$TE <- if_else(LONG22$TE <= 0, NA, LONG22$TE) # remove negatives and zeros on TE

LONG22$LEV <- LONG22$TD / LONG22$TE

# ICU = (AIO + RD) / NSL
Desc(LONG22$RD) # check negative values
Desc(LONG22$AIO) # check negative values / no negatives, correct
Desc(LONG22$TOE) # check negative and zero values

LONG22 <- LONG22 %>%  # change NA to 0
  mutate(RD = if_else(is.na(RD), 0, RD),
         AIO = if_else(is.na(AIO), 0, AIO))

# change negatives to NAs & remove zeros on TOE
LONG22 <- LONG22 %>%
  mutate(RD = if_else(RD < 0, NA, RD),
         AIO = if_else(AIO < 0, NA, AIO),
         TOE = if_else(TOE <= 0, NA, TOE))

LONG22$ICU <- (LONG22$AIO + LONG22$RD) / LONG22$TOE
Desc(LONG22$ICU)

# Filter out cases with NA in whole sample______________________________________
LONG22_DEL <- LONG22 %>%
  mutate(DEL = if_else(is.na(BEP) | is.na(ROA) | is.na(ROE) | is.na(SCR) | is.na(ICU) | is.na(LTA) | is.na(LFT) | is.na(WCR) | is.na(LEV), 1, 0))

LONG22_DEL <- LONG22_DEL %>%
  group_by(id) %>%
  summarise(DELS = sum(DEL))

LONG22_DEL <- LONG22_DEL %>%
  filter(DELS == 0)

PANEL22FILTERED <- LONG22 %>%
  filter(id %in% LONG22_DEL$id)

# Create panel dataframe after filtering NA and invalid cases___________________
PANEL22 <- pdata.frame(PANEL22FILTERED, index=c("id","wave"), drop.index = FALSE, row.names = TRUE)

# Winsorization & Descriptive statistics________________________________________
# BEP
PANEL22$BEP <- Winsorize(PANEL22$BEP, probs = c(0.02, 0.98))
Desc(PANEL22$BEP)

# ROA
PANEL22$ROA <- Winsorize(PANEL22$ROA, probs = c(0.02, 0.98))
Desc(PANEL22$ROA)

# ROE
PANEL22$ROE <- Winsorize(PANEL22$ROE, probs = c(0.02, 0.98))
Desc(PANEL22$ROE)

# SCR
PANEL22$SCR <- Winsorize(PANEL22$SCR, probs = c(0.02, 0.98))
Desc(PANEL22$SCR)

# ICU
PANEL22$ICU <- Winsorize(PANEL22$ICU, probs = c(0.02, 0.98))
Desc(PANEL22$ICU)

# LTA
PANEL22$LTA <- Winsorize(PANEL22$LTA, probs = c(0.02, 0.98))
Desc(PANEL22$LTA)

# LFT
PANEL22$LFT <- Winsorize(PANEL22$LFT, probs = c(0.02, 0.98))
Desc(PANEL22$LFT)

# WCR
PANEL22$WCR <- Winsorize(PANEL22$WCR, probs = c(0.02, 0.98))
Desc(PANEL22$WCR)

# LEV
PANEL22$LEV <- Winsorize(PANEL22$LEV, probs = c(0.02, 0.98))
Desc(PANEL22$LEV)

# Correlations__________________________________________________________________
cor.test(PANEL22$BEP, PANEL22$ROA)
cor.test(PANEL22$BEP, PANEL22$ROE)
cor.test(PANEL22$BEP, PANEL22$SCR)
cor.test(PANEL22$BEP, PANEL22$ICU)
cor.test(PANEL22$BEP, PANEL22$LTA)
cor.test(PANEL22$BEP, PANEL22$LFT)
cor.test(PANEL22$BEP, PANEL22$WCR)
cor.test(PANEL22$BEP, PANEL22$LEV)

cor.test(PANEL22$ROA, PANEL22$ROE)
cor.test(PANEL22$ROA, PANEL22$SCR)
cor.test(PANEL22$ROA, PANEL22$ICU)
cor.test(PANEL22$ROA, PANEL22$LTA)
cor.test(PANEL22$ROA, PANEL22$LFT)
cor.test(PANEL22$ROA, PANEL22$WCR)
cor.test(PANEL22$ROA, PANEL22$LEV)

cor.test(PANEL22$ROE, PANEL22$SCR)
cor.test(PANEL22$ROE, PANEL22$ICU)
cor.test(PANEL22$ROE, PANEL22$LTA)
cor.test(PANEL22$ROE, PANEL22$LFT)
cor.test(PANEL22$ROE, PANEL22$WCR)
cor.test(PANEL22$ROE, PANEL22$LEV)

cor.test(PANEL22$SCR, PANEL22$ICU)
cor.test(PANEL22$SCR, PANEL22$LTA)
cor.test(PANEL22$SCR, PANEL22$LFT)
cor.test(PANEL22$SCR, PANEL22$WCR)
cor.test(PANEL22$SCR, PANEL22$LEV)

cor.test(PANEL22$ICU, PANEL22$LTA)
cor.test(PANEL22$ICU, PANEL22$LFT)
cor.test(PANEL22$ICU, PANEL22$WCR)
cor.test(PANEL22$ICU, PANEL22$LEV)

cor.test(PANEL22$LTA, PANEL22$LFT)
cor.test(PANEL22$LTA, PANEL22$WCR)
cor.test(PANEL22$LTA, PANEL22$LEV)

cor.test(PANEL22$LFT, PANEL22$WCR)
cor.test(PANEL22$LFT, PANEL22$LEV)

cor.test(PANEL22$WCR, PANEL22$LEV)

# Correlations between lagged variables_________________________________________
# LAGS for correlation analysis
PANEL22$L1BEP <- lag(PANEL22$BEP, 1L)
PANEL22$L1ROA <- lag(PANEL22$ROA, 1L) 
PANEL22$L1ROE <- lag(PANEL22$ROE, 1L)
PANEL22$L1SCR <- lag(PANEL22$SCR, 1L)
PANEL22$L2SCR <- lag(PANEL22$SCR, 2L)
PANEL22$L1ICU <- lag(PANEL22$ICU, 1L)
PANEL22$L2ICU <- lag(PANEL22$ICU, 2L)

cor.test(PANEL22$SCR, PANEL22$L1SCR)
cor.test(PANEL22$SCR, PANEL22$L2SCR)
cor.test(PANEL22$L1SCR, PANEL22$L2SCR)

cor.test(PANEL22$ICU, PANEL22$L1ICU)
cor.test(PANEL22$ICU, PANEL22$L2ICU)
cor.test(PANEL22$L1ICU, PANEL22$L2ICU)

cor.test(PANEL22$SCR, PANEL22$L1ICU)
cor.test(PANEL22$SCR, PANEL22$L2ICU)

cor.test(PANEL22$L1SCR, PANEL22$ICU)
cor.test(PANEL22$L1SCR, PANEL22$L1ICU)
cor.test(PANEL22$L1SCR, PANEL22$L2ICU)

cor.test(PANEL22$L2SCR, PANEL22$ICU)
cor.test(PANEL22$L2SCR, PANEL22$L1ICU)
cor.test(PANEL22$L2SCR, PANEL22$L2ICU)

# Choose panel dataframe for estimation_________________________________________
PANEL22E <- PANEL22
# PANEL22E <- PANEL22 %>% filter(SECT == "Technology")
# PANEL22E <- PANEL22 %>% filter(SECT == "Healthcare")

PANEL22E_COUNTRY <- PANEL22E %>%
  group_by(CTRY, RIC) %>%
  tally()
PANEL22E_COUNTRY <- PANEL22E_COUNTRY %>%
  group_by(CTRY) %>%
  tally()

# Main estimation formulas: model (1)___________________________________________
fmodel <- formula(BEP ~ SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(BEP ~ SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROA ~ SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROA ~ SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROE ~ SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROE ~ SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

# First difference: model (2) without the intercept (-1)________________________
fmodelfd <- formula(BEP ~ SCR + ICU - 1) 

fmodelfd <- formula(ROA ~ SCR + ICU - 1)

fmodelfd <- formula(ROE ~ SCR + ICU - 1)

# Lag1 dependent estimation formulas (3)________________________________________
fmodel <- formula(BEP ~ lag(BEP) + SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(BEP ~ lag(BEP) + SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROA ~ lag(ROA) + SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROA ~ lag(ROA) + SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROE ~ lag(ROE) + SCR + ICU + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROE ~ lag(ROE) + SCR + ICU + LTA + LFT + WCR + LEV + factor(wave))

# Lag1 predictors estimation formulas (4)_______________________________________
fmodel <- formula(BEP ~ lag(BEP) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(BEP ~ lag(BEP) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROA ~ lag(ROA) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROA ~ lag(ROA) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROE ~ lag(ROE) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROE ~ lag(ROE) + lag(SCR) + lag(ICU) + LTA + LFT + WCR + LEV + factor(wave))

# Lag2 predictors estimation formulas (5)_______________________________________
fmodel <- formula(BEP ~ lag(BEP) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(BEP ~ lag(BEP) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROA ~ lag(ROA) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROA ~ lag(ROA) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV + factor(wave))

fmodel <- formula(ROE ~ lag(ROE) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV)
fmodel_years <- formula(ROE ~ lag(ROE) + lag(SCR, 2L) + lag(ICU, 2L) + LTA + LFT + WCR + LEV + factor(wave))

# Estimation results ___________________________________________________________
estpl <- plm(fmodel, data = PANEL22E, model = "pooling")
summary(estpl)
VIF(estpl)
plmtest(estpl, type=c("bp"))

estfd <- plm(fmodelfd, data = PANEL22E, model = "fd")
summary(estfd, vcov = vcovHC(estfd, method="arellano", type="HC1"))

estfe <- plm(fmodel, data = PANEL22E, model = "within")
summary(estfe, vcov = vcovHC(estfe, method="arellano", type="HC1"))
pwartest(estfe)

estols <- lm(fmodel, data = PANEL22E)
pFtest(estfe, estols) 

estfe_time <- plm(fmodel, data = PANEL22E, model = "within", effect = "twoways")
summary(estfe_time, vcov = vcovHC(estfe_time, method="arellano", type="HC1"))
fixef(estfe_time, "time")
pwartest(estfe_time)

estfe_years <- plm(fmodel_years, data = PANEL22E, model = "within")
summary(estfe_years, vcov = vcovHC)

pFtest(estfe_time, estfe)
plmtest(estfe, c("time"), type=("bp"))

estre <- plm(fmodel, data = PANEL22E, model = "random")

estre_time <- plm(fmodel, data = PANEL22E, model = "random", effect = "twoways", random.method = "walhus")

phtest(estfe, estre) #Hausmann test
phtest(estfe_time, estre_time) #Hausmann test

# System GMM Estimation in pdynmc_______________________________________________
data.info(
  PANEL22E,
  i.name = "id",
  t.name = "wave"
)

# Robustness test for model (3)
reg <- pdynmc(
  dat = PANEL22E, 
  varname.i = "id", varname.t = "wave",
  use.mc.diff = TRUE, use.mc.lev = FALSE, use.mc.nonlin = FALSE,
  include.y = TRUE, include.x = FALSE,
  varname.y = "ROE", lagTerms.y = 1, maxLags.y = 4,
  fur.con = TRUE, fur.con.diff = TRUE, fur.con.lev = FALSE,
  varname.reg.fur = c("SCR", "ICU", "LTA", "LFT", "WCR", "LEV"),
  lagTerms.reg.fur = c(0,0,0,0,0,0),
  include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, 
  varname.dum = "wave",
  w.mat = "iid.err", std.err = "corrected",
  estimation = "twostep", opt.meth = "none")
summary(reg)

# Robustness test for model (5)
reg <- pdynmc(
  dat = PANEL22E, 
  varname.i = "id", varname.t = "wave",
  use.mc.diff = TRUE, use.mc.lev = FALSE, use.mc.nonlin = FALSE,
  include.y = TRUE, include.x = FALSE,
  varname.y = "ROE", lagTerms.y = 1, maxLags.y = 4,
  fur.con = TRUE, fur.con.diff = TRUE, fur.con.lev = FALSE,
  varname.reg.fur = c("SCR", "ICU", "LTA", "LFT", "WCR", "LEV"),
  lagTerms.reg.fur = c(2,2,0,0,0,0),
  include.dum = TRUE, dum.diff = TRUE, dum.lev = FALSE, 
  varname.dum = "wave",
  w.mat = "iid.err", std.err = "corrected",
  estimation = "twostep", opt.meth = "none")
summary(reg)
